/*
 * 11/11/2001 - 17:58:01
 *
 * QRALocator.java - permet de la transformation entre un QRA locator et une coordonne gographiqy
 * Copyright (C) 2001 C. Mercier
 * c.avmdti@free.fr
 * www.avmdti.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/**
 * Titre : QRA LOCATOR<p>
 * Description : outil de calcul pour le QRA locator<p>
 * Copyright : Copyright (c) C. Mercier<p>
 * Socit : www.avmdti.org<p>
 * @author C. Mercier
 * @version 1.0
 */

package org.avmdti.qralocator;
/**
 * Class permettant de transformer des coordonnes QRA Locator en coordonnes
 * geographique et vice versa
 * @link coord.class
 */
public class QraLocator {

        static byte _J = new String("J").getBytes()[0];
        static byte _A = new String("A").getBytes()[0];
        static byte _0 = new String("0").getBytes()[0];
        final static double coef_lat = 0.025 * 100.0 / 60.0;
        final static double coef_long = 0.05 * 100.0 / 60.0;
        public QraLocator() {
        }

        /**
         * permet la tranformation de coordonnes gographiques en Locator
         * Les paramtres en entres sont des doubles
         * la forme est la suivante : XX.YYYYY exprim en degr dcimal
         * @param longitude
         * @param latitude
         * @return chaine de caractres contenant le locator
         */
        public String ToLocator(double longitude, double latitude) {
                String S = null;
                byte locator[] = new byte[10];
                // ajout pour les PB de prcisions sur les conversions
                longitude = 180 + longitude + 0.00001;
                latitude = 90 + latitude + 0.00001 ;
                // rcupration de la partie entire
                int _longitude = new Double(longitude).intValue();
                int _latitude = new Double(latitude).intValue();
                int _longitude1 = _longitude / 20;
                locator[0] = (byte)(_A + _longitude1);
                int _latitude1 = _latitude / 10;
                locator[1] = (byte)(_A + _latitude1);
                int _longitude2 = (_longitude - (_longitude1 * 20)) / 2;
                int _latitude2 = _latitude - (_latitude1 * 10);
                locator[2] = (byte)(_0 + _longitude2);
                locator[3] = (byte)(_0 + _latitude2);
                double _longitude3 = longitude -
                        ((_longitude1 * 20.0d) + (_longitude2 * 2.d));
                double _latitude3 = latitude -
                        ((_latitude1 * 10.0d) + (_latitude2 * 1.0d));
                locator[4] = (byte)(_A +
                        (int)(_longitude3 / (0.05 * 100.0 / 60.0)));
                locator[5] = (byte)(_A +
                        (int)(_latitude3 / (0.025 * 100.0 / 60.0)));
                S = new String (locator);
                return S;
        }
        /**
         * permet de calculer les coodonnes gographiques  partir des coordonnes
         * Locator
         * @param S : Qra locator en entre sous forme d'une chaine de caractre
         * @return : Objet de type coord
         */
        public coord FromLocator (String S) {
                coord C = null;
                String _S = S.toUpperCase();
                _S = _S.trim();
                byte t[] = _S.getBytes();
                double longitude, latitude;

                longitude = -180.0 + (t[0] - _A) * 20;
                latitude = -90.0 + (t[1] - _A) * 10;

                double L2 = 0;
                int v = (t[4] - _A);
                L2 = (v * coef_long) + coef_long / 2.0;
                longitude = longitude + (t[2] - _0) * 2 + L2;
                latitude = latitude + (t[3] - _0) +
                        (t[5] - _A) * (coef_lat) + coef_lat / 2.0 ;
                C = new coord(longitude, latitude);
                return C;

        }
}


